var manageReadWriteUtil = function(u2a, cmdInfo) {
  var regPage = document.querySelector('ti-widget-register-page');

  if (cmdInfo) {
    this.cmdQueue.push(cmdInfo);
  }
  if (this.readyToProcess && this.cmdQueue.length > 0) {
    var cmd = this.cmdQueue.shift();
    switch (cmd.type) {
      case 'write':
        console.log('Interface Util', 'Manage', 'Write', cmd);

        this.readyToProcess = false;
        this.manageWrites(u2a, [cmd.data]).then(
          function() {
            console.log('Interface Util', 'Manage', 'Write', 'Resolved');
            this.readyToProcess = true;
            this.manageReadWrite(u2a);
          }.bind(this)
        );
        break;

      case 'multipleWrite':
        console.log('Interface Util', 'Manage', 'Multiple write', cmd);

        this.readyToProcess = false;
        this.manageWrites(u2a, cmd.cmdArr).then(
          function() {
            if (cmd.promise) {
              cmd.promise.resolve();
            }

            console.log(
              'Interface Util',
              'Manage',
              'Multiple write',
              'Resolved'
            );
            this.readyToProcess = true;
            this.manageReadWrite(u2a);
          }.bind(this)
        );
        break;

      case 'commClear':
        // Applicable Only for SPI
        console.log('Interface Util', 'Manage', 'Comm clear', u2a);

        this.readyToProcess = false;
        u2a.sendCommandPacket(9, [1, 0x00]);
        this.readyToProcess = true;
        this.manageReadWrite(u2a);
        break;

      case 'read':
        console.log('Interface Util', 'Manage', 'Read', cmd);
        this.readyToProcess = false;

        if (typeof cmd.data[0] === 'number') {
          cmd.data = [cmd.data];
        }

        this.manageReads(u2a, cmd.data).then(
          function(resData) {
            cmd.promise.resolve(resData);

            console.log('Interface Util', 'Manage', 'Read', 'Resolve');

            this.readyToProcess = true;
            this.manageReadWrite(u2a);
          }.bind(this)
        );
        break;

      case 'poll':
        console.log('Interface Util', 'Manage', 'Poll', cmd);
        this.readyToProcess = false;

        // max bytes to read should be less than 136bytes in one transaction
        this.manageReads(u2a, [
          [0xc0, 0x05, 0x3d, 0x01, 0xfd, 0x75], // System Faults1,2.
          [0xc0, 0x05, 0x29, 0x02, 0xb2, 0x74], // Cell - Balancing Status
          [0xc0, 0x05, 0x9a, 0x01, 0x87, 0x45], // BAT Voltage.
          [0xc0, 0x05, 0x30, 0x02, 0xb9, 0xe4], // Device Status, fault_summary
          [0xc0, 0x05, 0x38, 0x01, 0xfe, 0x25], // Communication Faults - fault_comm1,2.
          [0xc0, 0x05, 0x40, 0x05, 0xdd, 0xe6], // Cells OVUV.
          [0xc0, 0x03, 0x18, 0x0f, 0x86, 0x20], // Cell - Balancing Timer
          [0xc0, 0x05, 0xa8, 0x1d, 0x93, 0xec] // GPIO Voltage gpi01-gpio16
        ]).then(resData1 => {
          this.manageReads(u2a, [
            [0xc0, 0x00, 0x1e, 0x05, 0xf5, 0x87], // Gpio_conf1-6
            [0xc0, 0x00, 0x3b, 0x01, 0xee, 0xd4], // Gpio_conf7,8
            [0xc0, 0x05, 0x74, 0x23, 0x4a, 0xfc], // Cell Voltage
            [0xc0, 0x05, 0x22, 0x00, 0x34, 0x85], // Bal_stat
            [0xc0, 0x07, 0x80, 0x00, 0xec, 0x25], // Debug Comm Stat
            [0xc0, 0x05, 0x94, 0x05, 0x82, 0xe6], // vf2,1 , CP
            [0xc0, 0x05, 0xc6, 0x04, 0x7e, 0x46], // cc_acc, cc_cnt
            [0xc0, 0x05, 0x55, 0x00, 0x13, 0x75], // fault_adc_misc
            [0xc0, 0x00, 0x11, 0x01, 0xf1, 0xb4] // Cs_adc_cal,2
          ]).then(resData2 => {
            resData1.push.apply(resData1, resData2);
            this.manageReads(u2a, [
              [0xc0, 0x05, 0x9c, 0x05, 0x85, 0x26], // CS 24bit, (cs_adc1,2-73x)
              [0xc0, 0x05, 0xea, 0x11, 0xa3, 0x49] // Die Temperature, Diagnostics
            ]).then(resData3 => {
              resData1.push(...resData3);
              console.log('Interface Util', 'Manage', 'Poll', 'Resolve');
              cmd.promise.resolve(resData1);
              this.readyToProcess = true;
              this.manageReadWrite(u2a);
            });
          });
        });
        break;

      case 'poll-600':
        // Applicable Only for 600
        console.log('Interface Util', 'Manage', 'Poll - 600', cmd);
        this.readyToProcess = false;

        // max bytes to read should be less than 136bytes in one transaction
        this.manageReads(u2a, [
          [0xa0, 0x05, 0x3d, 0x01, 0xe3, 0x75], // System Faults1,2.
          [0xa0, 0x05, 0x29, 0x02, 0xac, 0x74], // Cell - Balancing Status
          [0xa0, 0x05, 0x9a, 0x01, 0x99, 0x45], // BAT Voltage.
          [0xa0, 0x05, 0x30, 0x02, 0xa7, 0xe4], // Device Status,fault_summary
          [0xa0, 0x05, 0x38, 0x01, 0xe0, 0x25], // Communication Faults - fault_comm1,2.
          [0xa0, 0x05, 0x40, 0x05, 0xc3, 0xe6], // Cells OVUV.
          [0xa0, 0x05, 0xea, 0x11, 0xbd, 0x49], // Die Temperature, Diagnostics
          [0xa0, 0x03, 0x18, 0x0f, 0x98, 0x20], // Cell - Balancing Timer
          [0xa0, 0x05, 0xa8, 0x1d, 0x8d, 0xec] // GPIO Voltage gpio1-gpio15
        ]).then(resData1 => {
          this.manageReads(u2a, [
            [0xa0, 0x05, 0x74, 0x23, 0x54, 0xfc], // Cell Voltage.
            [0xa0, 0x05, 0x22, 0x00, 0x2a, 0x85], // Bal_stat.
            [0xa0, 0x07, 0x80, 0x00, 0xf2, 0x25], // Debug Comm Stat
            [0x80, 0x00, 0x21, 0x00, 0x00, 0x75, 0xd4], // 600 - Fault Summary
            [0x80, 0x00, 0x23, 0x00, 0x00, 0xd4, 0x14], // 600 - DEBUG_COMM_STAT
            [0xa0, 0x05, 0x94, 0x05, 0x9c, 0xe6], // vf2_hi/lo, vf1_hi/lo, CP
            [0xa0, 0x05, 0xc6, 0x04, 0x60, 0x46], // cc_acc, cc_cnt
            [0xa0, 0x05, 0x55, 0x00, 0x0d, 0x75], // fault_adc_misc
            [0xa0, 0x00, 0x11, 0x01, 0xef, 0xb4] // Cs_adc_cal1,2
          ]).then(resData2 => {
            resData1.push.apply(resData1, resData2);
            this.manageReads(u2a, [
              [0xa0, 0x00, 0x1e, 0x05, 0xeb, 0x87], // Gpio_conf1-6
              [0xa0, 0x05, 0x9c, 0x05, 0x9b, 0x26], // CS 24bit, (cs_adc1,2-73x)
              [0xa0, 0x00, 0x3b, 0x01, 0xf0, 0xd4] // Gpio_conf7,8
            ]).then(resData3 => {
              resData1.push(...resData3);
              cmd.promise.resolve(resData1);
              console.log('Interface Util', 'Manage', 'Poll - 600', 'Resolve');
              this.readyToProcess = true;
              this.manageReadWrite(u2a);
            });
          });
        });
        break;

      case 'autoAddress':
        console.log('Interface Util', 'Manage', 'Auto address', cmd);
        this.readyToProcess = false;

        // 616
        var writeData,
          writeDataArr = [],
          crc;
        // Broadcast 0x00 for ECC_TEST 0x350.
        writeDataArr.push([0xd0, 0x03, 0x50, 0x00, 0xf4, 0xe4]);
        // Broadcast 0x01 for CONTROL1 0x309.
        writeDataArr.push([0xd0, 0x03, 0x09, 0x01, 0x0f, 0x74]);
        // Broadcast addr for DIR0_ADDR 0x306
        for (var i = 0; i < 36; i++) {
          writeData = [0xd0, 0x03, 0x06, i];
          crc = getByteArrFromHexStr(
            prependZero(getCRC16IBM(writeData).toString(16), 4)
          );
          writeData.push.apply(writeData, crc);
          writeDataArr.push(writeData);
        }

        this.manageWrites(u2a, writeDataArr).then(
          function() {
            this.recursivePing(u2a, 0).then(
              function(noOfDevices) {
                console.log('UART', 'No. of devices', noOfDevices);
                this.noOfDevices = noOfDevices;

                writeDataArr = [];
                // Broadcast write 0x00 for COMM_CTRL 0x308
                writeDataArr.push([0xd0, 0x03, 0x08, 0x00, 0xcf, 0x24]);
                if (noOfDevices > 0) {
                  // Single device 0x01 for COMM_CTRL 0x308 at stack top
                  writeData = [0x90, noOfDevices - 1, 0x03, 0x08, 0x01];
                  crc = getByteArrFromHexStr(
                    prependZero(getCRC16IBM(writeData).toString(16), 4)
                  );
                  writeData.push.apply(writeData, crc);
                  writeDataArr.push(writeData);
                }

                this.manageWrites(u2a, writeDataArr).then(
                  function() {
                    this.manageReads(u2a, [
                      [0xc0, 0x03, 0x50, 0x00, 0xf0, 0x24], // Broadcast read ECC_TEST 0x350.
                      [0xc0, 0x03, 0x06, 0x00, 0xcf, 0x84] // Broadcast read DIR0_ADDR 0x306.
                    ]).then(
                      function() {
                        cmd.promise.resolve(noOfDevices);

                        console.log(
                          'Interface Util',
                          'Manage',
                          'Auto address',
                          'Resolve'
                        );
                        this.readyToProcess = true;
                        this.manageReadWrite(u2a);
                      }.bind(this)
                    );
                  }.bind(this)
                );
              }.bind(this)
            );
          }.bind(this)
        );
        break;

      case 'reverse-autoaddress':
        console.log('Interface Util', 'Manage', 'Reverse auto address', cmd);
        this.readyToProcess = false;

        // 616
        var writeData,
          writeDataArr = [],
          crc;
        // Broadcast 0x00 for ECC_TEST 0x350
        // u2a.sendCommandPacket(145, [0xD0, 0x03, 0x4C, 0x00, 0xFC, 0x24]);
        writeDataArr.push([0xd0, 0x03, 0x50, 0x00, 0xf4, 0xe4]);
        // Single device 0x80 for CONTROL1 0x309 at 0x00.
        writeDataArr.push([0x90, 0x00, 0x03, 0x09, 0x80, 0x13, 0xed]);
        // Reverse broadcast 0x80 for CONTROL1 0x309
        writeDataArr.push([0xe0, 0x03, 0x09, 0x80, 0xc0, 0x14]);
        // Broadcast 0x81 for CONTROL1 0x309.
        writeDataArr.push([0xd0, 0x03, 0x09, 0x81, 0x0e, 0xd4]);
        // Broadcast addr for DIR1_ADDR 0x307
        for (var i = 0; i < 36; i++) {
          writeData = [0xd0, 0x03, 0x07, i];
          crc = getByteArrFromHexStr(
            prependZero(getCRC16IBM(writeData).toString(16), 4)
          );
          writeData.push.apply(writeData, crc);
          writeDataArr.push(writeData);
        }

        this.manageWrites(u2a, writeDataArr).then(
          function() {
            this.recursivePing(u2a, 0).then(
              function(noOfDevices) {
                console.log('Interface Util', 'No. of devices', noOfDevices);
                this.noOfDevices = noOfDevices;

                writeDataArr = [];

                // Broadcast write 0x00 for COMM_CTRL 0x308 at 0x00
                writeDataArr.push([0xd0, 0x03, 0x08, 0x00, 0xcf, 0x24]);
                if (noOfDevices > 0) {
                  // Single device 0x03 for COMM_CTRL 0x308 at stack top
                  writeData = [0x90, noOfDevices - 1, 0x03, 0x08, 0x01];
                  crc = getByteArrFromHexStr(
                    prependZero(getCRC16IBM(writeData).toString(16), 4)
                  );
                  writeData.push.apply(writeData, crc);
                  writeDataArr.push(writeData);
                }

                this.manageWrites(u2a, writeDataArr).then(
                  function() {
                    this.manageReads(u2a, [
                      [0xc0, 0x03, 0x50, 0x00, 0xf0, 0x24], // Broadcast read ECC_TEST 0x34C.
                      [0xc0, 0x03, 0x07, 0x00, 0xce, 0x14] // Broadcast read DIR0_ADDR 0x306.
                    ]).then(
                      function() {
                        cmd.promise.resolve(noOfDevices);

                        console.log(
                          'Interface Util',
                          'Manage',
                          'Auto address',
                          'Resolve'
                        );
                        this.readyToProcess = true;
                        this.manageReadWrite(u2a);
                      }.bind(this)
                    );
                  }.bind(this)
                );
              }.bind(this)
            );
          }.bind(this)
        );
        break;

      case 'autoAddress-600':
        console.log('Interface Util', 'Manage', 'Auto address - 600', cmd);
        this.readyToProcess = false;

        // 600
        var writeData,
          writeDataArr = [],
          crc;
        // Single device 0x14 for DEV_CONF1 0x2001 at 0x00
        writeDataArr.push([0x90, 0x00, 0x20, 0x01, 0x14, 0xe4, 0x48]);

        this.manageWrites(u2a, writeDataArr).then(
          function() {
            // setTimeout(function() {
            writeDataArr = [];
            // Broadcast 0x00 for ECC_TEST 0x350
            writeDataArr.push([0xd0, 0x03, 0x50, 0x00, 0xf4, 0xe4]);
            // Broadcast 0x01 for CONTROL1 0x0309
            writeDataArr.push([0xd0, 0x03, 0x09, 0x01, 0x0f, 0x74]);
            // Broadcast addr for DIR0_ADDR 0x306
            for (var i = 0; i < 36; i++) {
              writeData = [0xd0, 0x03, 0x06, i];
              crc = getByteArrFromHexStr(
                prependZero(getCRC16IBM(writeData).toString(16), 4)
              );
              writeData.push.apply(writeData, crc);
              writeDataArr.push(writeData);
            }

            this.manageWrites(u2a, writeDataArr).then(
              function() {
                this.recursivePing(u2a, 0).then(
                  function(noOfDevices) {
                    console.log(
                      'Interface Util',
                      'No. of devices',
                      noOfDevices
                    );
                    this.noOfDevices = noOfDevices;

                    writeDataArr = [];
                    // Single device write to devices 1, 2, 3… address 0x308 = 0x02
                    for (var dev = 1; dev < noOfDevices; dev++) {
                      writeData = [0x90, dev, 0x03, 0x08, 0x00];
                      crc = getByteArrFromHexStr(
                        prependZero(getCRC16IBM(writeData).toString(16), 4)
                      );
                      writeData.push.apply(writeData, crc);
                      writeDataArr.push(writeData);
                    }
                    if (noOfDevices > 1) {
                      // Single device 0x03 for COMM_CTRL 0x308 at stack top
                      writeData = [0x90, noOfDevices - 1, 0x03, 0x08, 0x01];
                      crc = getByteArrFromHexStr(
                        prependZero(getCRC16IBM(writeData).toString(16), 4)
                      );
                      writeData.push.apply(writeData, crc);
                      writeDataArr.push(writeData);
                    }

                    this.manageWrites(u2a, writeDataArr).then(
                      function() {
                        this.manageReads(u2a, [
                          [0xa0, 0x03, 0x50, 0x00, 0xee, 0x24], // Stack read ECC_TEST 0x306
                          [0x80, 0x00, 0x20, 0x01, 0x00, 0x25, 0x84], // Single read read DEV_CONF1 0x2001 at 0x00
                          [0xa0, 0x03, 0x06, 0x00, 0xd1, 0x84] // Stack read DIR0_ADDR 0x306
                        ]).then(
                          function() {
                            writeDataArr = [];
                            // Stack 0x03 for FAULT_RST2 0x332
                            // Single device 0x00 for COMM_TIMEOUT 0x2005 at 0x00
                            writeDataArr.push([
                              0x90,
                              0x00,
                              0x20,
                              0x05,
                              0x00,
                              0xe6,
                              0x87
                            ]);

                            this.manageWrites(u2a, writeDataArr).then(
                              function() {
                                cmd.promise.resolve(noOfDevices);

                                console.log(
                                  'Interface Util',
                                  'Manage',
                                  'Auto address- 600',
                                  'Resolve'
                                );
                                this.readyToProcess = true;
                                this.manageReadWrite(u2a);
                              }.bind(this)
                            );
                          }.bind(this)
                        );
                      }.bind(this)
                    );
                  }.bind(this)
                );
              }.bind(this)
            );
          }.bind(this)
        );
        break;

      case 'reverse-autoaddress-600':
        console.log(
          'Interface Util',
          'Manage',
          'Reverse auto address - 600',
          cmd
        );
        this.readyToProcess = false;

        var writeData,
          writeDataArr = [],
          crc;
        //Single device write to 600 CONTROL1[DIR_SEL]=1
        writeDataArr.push([0x90, 0x00, 0x03, 0x09, 0x80, 0x13, 0xed]);
        //Single device write to 600 DEV_CONF1[SEL_TX_TONE]=0
        writeDataArr.push([0x90, 0x00, 0x20, 0x01, 0x14, 0xe4, 0x48]);

        this.manageWrites(u2a, writeDataArr).then(
          function() {
            writeDataArr = [];
            // Broadcast 0x00 for ECC_TEST 0x350
            writeDataArr.push([0xd0, 0x03, 0x50, 0x00, 0xf4, 0xe4]);
            //Broadcast write reverse address 0x309 = 0x80 (change stack devices direction DIR_SEL=1)
            writeDataArr.push([0xe0, 0x03, 0x09, 0x80, 0xc0, 0x14]);
            //Broadcast write address 0x308 = 0x02
            writeDataArr.push([0xd0, 0x03, 0x08, 0x00, 0xcf, 0x24]);
            //Broadcast write address 0x309 = 0x81 (enable 616 auto addressing)
            writeDataArr.push([0xd0, 0x03, 0x09, 0x81, 0x0e, 0xd4]);
            //Broadcast write consecutively address 0x307 = 0, 1, 2, 3… (0 base, rest is stack)
            for (var i = 0; i < 66; i++) {
              writeData = [0xd0, 0x03, 0x07, i];
              crc = getByteArrFromHexStr(
                prependZero(getCRC16IBM(writeData).toString(16), 4)
              );
              writeData.push.apply(writeData, crc);
              writeDataArr.push(writeData);
            }

            this.manageWrites(u2a, writeDataArr).then(
              function() {
                this.recursivePing(u2a, 0).then(
                  function(noOfDevices) {
                    console.log(
                      'Interface Util',
                      'No. of devices',
                      noOfDevices
                    );
                    this.noOfDevices = noOfDevices;

                    writeDataArr = [];
                    for (var dev = 1; dev < noOfDevices; dev++) {
                      writeData = [0x90, dev, 0x03, 0x08, 0x00];
                      crc = getByteArrFromHexStr(
                        prependZero(getCRC16IBM(writeData).toString(16), 4)
                      );
                      writeData.push.apply(writeData, crc);
                      writeDataArr.push(writeData);
                    }
                    if (noOfDevices > 1) {
                      // Single device 0x03 for COMM_CTRL 0x308 at stack top
                      writeData = [0x90, noOfDevices - 1, 0x03, 0x08, 0x01];
                      crc = getByteArrFromHexStr(
                        prependZero(getCRC16IBM(writeData).toString(16), 4)
                      );
                      writeData.push.apply(writeData, crc);
                      writeDataArr.push(writeData);
                    }

                    this.manageWrites(u2a, writeDataArr).then(
                      function() {
                        this.manageReads(u2a, [
                          [0xa0, 0x03, 0x50, 0x00, 0xee, 0x24], // Stack read OTC_ECC_TEST  0x34C
                          [0xa0, 0x03, 0x07, 0x00, 0xd0, 0x14], // Stack read address 0x307 (read back to verify addresses are correct)
                          [0x80, 0x00, 0x20, 0x01, 0x00, 0x25, 0x84] // Single read DEV_CONF1 0x2001 at 0x00
                        ]).then(
                          function() {
                            writeDataArr = [];
                            // Single device 0x00 for COMM_TIMEOUT 0x2005 at 0x00
                            writeDataArr.push([
                              0x90,
                              0x00,
                              0x20,
                              0x05,
                              0x00,
                              0xe6,
                              0x87
                            ]);

                            this.manageWrites(u2a, writeDataArr).then(
                              function() {
                                cmd.promise.resolve(noOfDevices);

                                console.log(
                                  'Interface Util',
                                  'Manage',
                                  'Reverse auto address - 600',
                                  'Resolve'
                                );
                                this.readyToProcess = true;
                                this.manageReadWrite(u2a);
                              }.bind(this)
                            );
                          }.bind(this)
                        );
                      }.bind(this)
                    );
                  }.bind(this)
                );
              }.bind(this)
            );
          }.bind(this)
        );
        break;

      case 'readAllBroadcast':
        console.log('UART', 'Manage', 'Read all broadcast', cmd);
        this.readyToProcess = false;

        var consolidatedData = [],
          startAddr = 0,
          readData,
          crc;

        function recursive128BroadcastRead() {
          readData = [];

          readData.push(0xc0);
          readData.push.apply(
            readData,
            getByteArrFromHexStr(prependZero(startAddr.toString(16), 4))
          );
          readData.push(0x7f);
          crc = getByteArrFromHexStr(
            prependZero(getCRC16IBM(readData).toString(16), 4)
          );
          readData.push.apply(readData, crc);

          startAddr += 128;

          this.manageReads(u2a, [readData]).then(
            function(resData) {
              var data = parseDeviceResponse(resData);
              consolidatedData.push.apply(consolidatedData, data);

              if (startAddr < 2048) {
                recursive128BroadcastRead.bind(this)();
              } else {
                console.log(
                  'UART',
                  'Interface',
                  'Read all',
                  'Consolidated data',
                  consolidatedData
                );

                var regDataArr,
                  regDataRowIndex,
                  regDataColumnIndex,
                  regAddr,
                  rejectFlag;

                var resolveCount = 0;

                for (var i = 0; i < cmd.promiseArr.length; i++) {
                  regAddr = cmd.promiseArr[i].addr;
                  regDataRowIndex =
                    Math.floor(regAddr / 128) * this.noOfDevices;
                  regDataColumnIndex = regAddr % 128;
                  regDataArr = [];
                  rejectFlag = false;
                  for (var j = 0; j < this.noOfDevices; j++) {
                    if (consolidatedData[regDataRowIndex + j]) {
                      regDataArr.push(
                        consolidatedData[regDataRowIndex + j][
                          regDataColumnIndex + 4
                        ]
                      );
                    } else {
                      rejectFlag = true;
                    }
                  }
                  if (rejectFlag) {
                    cmd.promiseArr[i].promise.reject();
                  } else {
                    cmd.promiseArr[i].promise.resolve(regDataArr.reverse());
                    resolveCount++;
                    console.log(
                      'Interface Util',
                      'Interface',
                      'Read all - Broadcast',
                      cmd.promiseArr[i].addr,
                      regDataArr
                    );
                  }
                }

                console.log(
                  'Interface Util',
                  'Interface',
                  'Read all - Broadcast',
                  'Done',
                  resolveCount
                );

                regReadDetail = undefined;

                regPage.isUpdating = false;
                console.log(
                  'Interface Util',
                  'Manage',
                  'Read all broadcast',
                  'Regmap - Hide progress',
                  regPage
                );

                console.log(
                  'Interface Util',
                  'Manage',
                  'Read all broadcast',
                  'Resolve'
                );

                this.readyToProcess = true;
                this.manageReadWrite(u2a);
              }
            }.bind(this)
          );
        }

        recursive128BroadcastRead.bind(this)();

        break;

      case 'readAllStack':
        console.log('Interface Util', 'Manage', 'Read all stack', cmd);
        this.readyToProcess = false;

        var consolidatedData = [],
          startAddr = 0,
          readData,
          crc;

        function recursive128StackRead() {
          readData = [];

          readData.push(0xa0);
          readData.push.apply(
            readData,
            getByteArrFromHexStr(prependZero(startAddr.toString(16), 4))
          );
          readData.push(0x7f);
          crc = getByteArrFromHexStr(
            prependZero(getCRC16IBM(readData).toString(16), 4)
          );
          readData.push.apply(readData, crc);

          startAddr += 128;

          this.manageReads(u2a, [readData]).then(
            function(resData) {
              var data = parseDeviceResponse(resData);
              consolidatedData.push.apply(consolidatedData, data);

              if (startAddr < 2048) {
                recursive128StackRead.bind(this)();
              } else {
                console.log(
                  'Interface Util',
                  'Interface',
                  'Read all - Stack',
                  'Consolidated data',
                  consolidatedData
                );

                var regDataArr,
                  regDataRowIndex,
                  regDataColumnIndex,
                  regAddr,
                  rejectFlag;

                var resolveCount = 0;

                for (var i = 0; i < cmd.promiseArr.length; i++) {
                  regAddr = cmd.promiseArr[i].addr;
                  regDataRowIndex =
                    Math.floor(regAddr / 128) * (this.noOfDevices - 1);
                  regDataColumnIndex = regAddr % 128;
                  regDataArr = [];
                  rejectFlag = false;
                  for (var j = 0; j < this.noOfDevices - 1; j++) {
                    if (consolidatedData[regDataRowIndex + j]) {
                      regDataArr.push(
                        consolidatedData[regDataRowIndex + j][
                          regDataColumnIndex + 4
                        ]
                      );
                    } else {
                      rejectFlag = true;
                    }
                  }
                  if (rejectFlag) {
                    cmd.promiseArr[i].promise.reject();
                  } else {
                    cmd.promiseArr[i].promise.resolve(regDataArr.reverse());
                    resolveCount++;
                    console.log(
                      'Interface Util',
                      'Interface',
                      'Read all - Stack',
                      cmd.promiseArr[i].addr,
                      regDataArr
                    );
                  }
                }

                console.log(
                  'Interface Util',
                  'Interface',
                  'Read all - Stack',
                  'Done',
                  resolveCount
                );

                regReadDetail = undefined;

                regPage.isUpdating = false;
                console.log(
                  'Interface Util',
                  'Manage',
                  'Read all stack',
                  'Regmap - Hide progress',
                  regPage
                );

                console.log(
                  'Interface Util',
                  'Manage',
                  'Read all stack',
                  'Resolve'
                );

                this.readyToProcess = true;
                this.manageReadWrite(u2a);
              }
            }.bind(this)
          );
        }

        recursive128StackRead.bind(this)();

        break;

      case 'readAllSingle':
        console.log('Interface Util', 'Manage', 'Read all single', cmd);
        this.readyToProcess = false;

        var consolidatedData = [],
          startAddr = 0,
          readData,
          crc;

        function recursive128SingleRead() {
          readData = [];

          readData.push(0x80);
          readData.push(cmd.device);
          readData.push.apply(
            readData,
            getByteArrFromHexStr(prependZero(startAddr.toString(16), 4))
          );
          readData.push(0x7f);
          crc = getByteArrFromHexStr(
            prependZero(getCRC16IBM(readData).toString(16), 4)
          );
          readData.push.apply(readData, crc);

          startAddr += 128;

          this.manageReads(u2a, [readData]).then(
            function(resData) {
              var data = parseDeviceResponse(resData);
              consolidatedData.push.apply(consolidatedData, data);

              if (startAddr < 2048) {
                recursive128SingleRead.bind(this)();
              } else {
                console.log(
                  'Interface Util',
                  'Interface',
                  'Read all - Single',
                  'Consolidated data',
                  consolidatedData
                );

                var regDataRowIndex, regDataColumnIndex, regAddr;

                var resolveCount = 0;

                for (var i = 0; i < cmd.promiseArr.length; i++) {
                  regAddr = cmd.promiseArr[i].addr;
                  regDataRowIndex = Math.floor(regAddr / 128);
                  regDataColumnIndex = regAddr % 128;
                  if (consolidatedData[regDataRowIndex]) {
                    cmd.promiseArr[i].promise.resolve(
                      consolidatedData[regDataRowIndex][regDataColumnIndex + 4]
                    );
                    resolveCount++;
                    console.log(
                      'Interface Util',
                      'Interface',
                      'Read all - Single',
                      cmd.promiseArr[i].addr,
                      consolidatedData[regDataRowIndex][regDataColumnIndex + 4]
                    );
                  } else {
                    cmd.promiseArr[i].promise.reject();
                  }
                }

                console.log(
                  'Interface Util',
                  'Interface',
                  'Read all - Single',
                  'Done',
                  resolveCount
                );

                regReadDetail = undefined;

                regPage.isUpdating = false;
                console.log(
                  'Interface Util',
                  'Manage',
                  'Read all single',
                  'Regmap - Hide progress',
                  regPage
                );

                console.log(
                  'Interface Util',
                  'Manage',
                  'Read all single',
                  'Resolve'
                );

                this.readyToProcess = true;
                this.manageReadWrite(u2a);
              }
            }.bind(this)
          );
        }

        recursive128SingleRead.bind(this)();

        break;

      case 'readAllSingle600':
        console.log('Interface Util', 'Manage', 'Read all single - 600', cmd);
        this.readyToProcess = false;

        var consolidatedData = [],
          startAddr = 8192,
          readData,
          crc;

        function recursive128SingleRead600() {
          readData = [];

          readData.push(0x80);
          readData.push(cmd.device);
          readData.push.apply(
            readData,
            getByteArrFromHexStr(prependZero(startAddr.toString(16), 4))
          );
          readData.push(0x7f);
          crc = getByteArrFromHexStr(
            prependZero(getCRC16IBM(readData).toString(16), 4)
          );
          readData.push.apply(readData, crc);

          startAddr += 128;

          this.manageReads(u2a, [readData]).then(
            function(resData) {
              var data = parseDeviceResponse(resData);
              consolidatedData.push.apply(consolidatedData, data);

              if (startAddr < 9216) {
                recursive128SingleRead600.bind(this)();
              } else {
                console.log(
                  'Interface Util',
                  'Interface',
                  'Read all - 600',
                  'Consolidated data',
                  consolidatedData
                );

                var regDataRowIndex, regDataColumnIndex, regAddr;

                var resolveCount = 0;

                for (var i = 0; i < cmd.promiseArr.length; i++) {
                  regAddr = cmd.promiseArr[i].addr - 8192;
                  regDataRowIndex = Math.floor(regAddr / 128);
                  regDataColumnIndex = regAddr % 128;
                  if (consolidatedData[regDataRowIndex]) {
                    cmd.promiseArr[i].promise.resolve(
                      consolidatedData[regDataRowIndex][regDataColumnIndex + 4]
                    );
                    resolveCount++;
                    console.log(
                      'Interface Util',
                      'Interface',
                      'Read all - 600',
                      cmd.promiseArr[i].addr,
                      consolidatedData[regDataRowIndex][regDataColumnIndex + 4]
                    );
                  } else {
                    cmd.promiseArr[i].promise.reject();
                  }
                }

                readData = [0x80, cmd.device, 0x03, 0x06, 0x04];
                crc = getByteArrFromHexStr(
                  prependZero(getCRC16IBM(readData).toString(16), 4)
                );
                readData.push.apply(readData, crc);

                this.manageReads(u2a, [readData]).then(
                  function(resData2) {
                    var data = resData2;
                    console.log(
                      'Interface Util',
                      'Interface',
                      'Read all - 600',
                      'Consolidated data 2',
                      data
                    );
                    var addr, val;

                    for (var i = 0; i < cmd.promiseArr.length; i++) {
                      addr = cmd.promiseArr[i].addr;

                      if (addr > 0x305 && addr < 0x30b) {
                        val = data[addr - 0x306 + 4];
                        if (val) {
                          cmd.promiseArr[i].promise.resolve(val);
                        } else {
                          cmd.promiseArr[i].promise.reject();
                        }
                        resolveCount++;
                        console.log(
                          'Interface Util',
                          'Interface',
                          'Read all - 600',
                          cmd.promiseArr[i].addr,
                          val
                        );
                      }
                    }

                    console.log(
                      'Interface Util',
                      'Interface',
                      'Read all - Single - 600',
                      'Done',
                      resolveCount
                    );

                    regReadDetail = undefined;

                    regPage.isUpdating = false;
                    console.log(
                      'Interface Util',
                      'Manage',
                      'Read all single - 600',
                      'Regmap - Hide progress',
                      regPage
                    );

                    console.log(
                      'Interface Util',
                      'Manage',
                      'Read all single - 600',
                      'Resolve'
                    );

                    this.readyToProcess = true;
                    this.manageReadWrite(u2a);
                  }.bind(this)
                );
              }
            }.bind(this)
          );
        }

        recursive128SingleRead600.bind(this)();

        break;

      case 'cb-remaining-time':
        var remainingTimeArr = [];
        this.readyToProcess = false;
        function readRemainingTime(cellNo) {
          writeDataArr = [];
          writeData = [0x90, cmd.device, 0x03, 0x3a, cellNo * 2 + 1];
          crc = getByteArrFromHexStr(
            prependZero(getCRC16IBM(writeData).toString(16), 4)
          );
          writeData.push.apply(writeData, crc);
          writeDataArr.push(writeData);
          this.manageWrites(u2a, writeDataArr).then(
            function() {
              readData = [0x80, cmd.device, 0x05, 0x2c, 0x00];
              crc = getByteArrFromHexStr(
                prependZero(getCRC16IBM(readData).toString(16), 4)
              );
              readData.push.apply(readData, crc);

              this.manageReads(u2a, [readData]).then(
                function(data) {
                  remainingTimeArr.push(data[4]);
                  if (cellNo < cmd.noOfCells - 1) {
                    readRemainingTime.bind(this)(cellNo + 1);
                  } else {
                    cmd.promise.resolve(remainingTimeArr);
                    remainingTimeArr = [];
                    this.readyToProcess = true;
                    this.manageReadWrite(u2a);
                  }
                }.bind(this)
              );
            }.bind(this)
          );
        }

        readRemainingTime.bind(this)(0);

        break;
    }
  }
};
